import java.io.*;
import java.net.*;
import java.util.*;

class MainServer  {

    BufferedReader br;
    PrintStream ps;
    ServerSocket conn;
    Socket sock;
    int port;
    ArrayList<Position> positions;

    public MainServer(int port) throws IOException {
	this.port = port;
	conn = new ServerSocket(port);
    }
  
    public void mainLoop() throws IOException {

	while(true) {
	    sock = conn.accept();
	    br = new BufferedReader(new InputStreamReader(sock.getInputStream()));
	    ps = new PrintStream(sock.getOutputStream());
	    positions = new ArrayList<Position>();
	    requestLoop();
	}
    }


    public void requestLoop() {

	String numReq = "";
	try {
	    while(true) {
		numReq = br.readLine();
		if (numReq == null) { // violent close from client
		    break;
		}
		else if (numReq == "") { // explicit close
		    break;
		}
		if (numReq.equals("1")) {
		    requestStorePos();
		}
		else if (numReq.equals("2")) {
		    requestGetPathLength();
		}
		else if (numReq.equals("3")) {
		    requestBeingThere();
		}		
	    }
	}
	catch(IOException e) {
	    System.out.println("problem with receiving request: "+e.getMessage());	    
	}
    }

    protected void requestStorePos() throws IOException {
	System.out.println("processing request STORE POSITION");
	

	boolean err = false;
	double x=0,y=0,z=0;	
	String params = br.readLine();
	String[] lst = params.split(",");
	if (lst.length != 3) {
	    err = true;
	}
	else {
	    try {
		x = Double.parseDouble(lst[0]);
		y = Double.parseDouble(lst[1]);
		z = Double.parseDouble(lst[2]);
	    }
	    catch(NumberFormatException e) {
		err = true;
	    }
	}
	if (err) {
	    ps.println("REQ_ERR");
	}
	else {
	    positions.add(new Position(x,y,z));
	    ps.println("OK");
	}
    }

    protected void requestGetPathLength() throws IOException {
	System.out.println("processing request GET PATH LENGTH");
	
	double length = 0.0;
	boolean first = true;
	Position pOld = null;
	for(Position p : positions) {
	    if (first) {
		pOld = p;
		first = false;
	    }
	    else {
		length += pOld.distance(p);
		pOld = p;
	    }
	}
	ps.println(length);
    }

    protected void requestBeingThere() throws IOException {
	System.out.println("processing request BEING THERE");

	boolean err = false;
	double prec=0,x=0,y=0,z=0;	
	String params = br.readLine();
	String[] lst = params.split(",");
	if (lst.length != 4) {
	    err = true;
	}
	else {
	    try {
		prec = Double.parseDouble(lst[0]);
		x = Double.parseDouble(lst[1]);
		y = Double.parseDouble(lst[2]);
		z = Double.parseDouble(lst[3]);
	    }
	    catch(NumberFormatException e) {
		err = true;
	    }
	}
	if (err) {
	    ps.println("REQ_ERR");
	}
	else {
	    Position loc = new Position(x,y,z);
	    boolean ok = false;
	    for (Position p : positions) {
		if (loc.equals(p,prec)) {
		    ok = true;
		    break;
		}
	    }
	    if (ok) {
		ps.println("TRUE");
	    }
	    else {
		ps.println("FALSE");		
	    }
	}
    }    
}

		
