import java.io.*;
import java.net.*;
import java.util.*;

class MainServer  {

    BufferedReader br;
    PrintStream ps;
    ServerSocket conn;
    Socket sock;
    int port;
    HashMap<Integer, ArrayList<Position> > positions;
    int idClient;

    public MainServer(int port) throws IOException {
	this.port = port;
	conn = new ServerSocket(port);
	idClient = 1;
	positions = new HashMap<Integer, ArrayList<Position> >();
    }
  
    public void mainLoop() throws IOException {

	while(true) {
	    sock = conn.accept();
	    br = new BufferedReader(new InputStreamReader(sock.getInputStream()));
	    ps = new PrintStream(sock.getOutputStream());
	    // create list for idClient
	    ArrayList<Position> pos = new ArrayList<>();
	    positions.put(idClient,pos);
	    requestLoop();
	}
    }

    public void requestLoop() {

	String numReq = "";
	String idStr = "";
	int id = -1;	
	try {

	    // sending id to client
	    ps.println(idClient++); 

	    while(true) {
		numReq = br.readLine();
		if (numReq == null) { // violent close from client
		    break;
		}
		else if (numReq == "") { // explicit close
		    break;
		}

		//reading and testing id sent by client
		idStr = br.readLine();
		boolean err = false;
		try {
		    id = Integer.parseInt(idStr);
		}
		catch(NumberFormatException e) {
		    err = true;
		}

		if (err) {
		    ps.println("REQ_ERR");	
		}
		else if (!positions.containsKey(id)) {
		    ps.println("NOID_ERR");
		}
		else if (numReq.equals("1")) {
		    requestStorePos(id);
		}
		else if (numReq.equals("2")) {
		    requestGetPathLength(id);
		}
		else if (numReq.equals("3")) {
		    requestFindPos(id);
		}		
	    }
	}
	catch(IOException e) {
	    System.out.println("problem with receiving request: "+e.getMessage());	    
	}
    }

    protected void requestStorePos(int id) throws IOException {
	System.out.println("processing request STORE POSITION");      

	boolean err = false;
	double x=0,y=0,z=0;	
	String params = br.readLine();
	String[] lst = params.split(",");
	if (lst.length != 3) {
	    err = true;
	}
	else {
	    try {
		x = Double.parseDouble(lst[0]);
		y = Double.parseDouble(lst[1]);
		z = Double.parseDouble(lst[2]);
	    }
	    catch(NumberFormatException e) {
		err = true;
	    }
	}
	if (err) {
	    ps.println("REQ_ERR");
	    return;
	}

	ArrayList<Position> pos = positions.get(id);
	pos.add(new Position(x,y,z));
	ps.println("OK");
    }

    protected void requestGetPathLength(int id) throws IOException {
	System.out.println("processing request GET PATH LENGTH");


	double length = 0.0;
	boolean first = true;
	Position pOld = null;
	boolean err = false;

	ArrayList<Position> pos = positions.get(id);
	
	for(Position p : pos) {
	    if (first) {
		pOld = p;
		first = false;
	    }
	    else {
		length += pOld.distance(p);
		pOld = p;
	    }
	}
	ps.println(length);
    }

    protected void requestFindPos(int id) throws IOException {
	System.out.println("processing request FIND POSITION");

	boolean err = false;
	double prec=0,x=0,y=0,z=0;	
	String params = br.readLine();
	String[] lst = params.split(",");
	
	if (lst.length != 4) {
	    err = true;
	}
	else {
	    try {
		prec = Double.parseDouble(lst[0]);		
		x = Double.parseDouble(lst[1]);
		y = Double.parseDouble(lst[2]);
		z = Double.parseDouble(lst[3]);
	    }
	    catch(NumberFormatException e) {
		err = true;
	    }
	}
	if (err) {
	    ps.println("REQ_ERR");
	    return;
	}

	ArrayList<Position> pos = positions.get(id);	
	Position loc = new Position(x,y,z);
	boolean ok = false;
	for (Position p : pos) {
	    if (loc.equals(p,prec)) {
		ok = true;
		break;
	    }
	}
	if (ok) {
	    ps.println("TRUE");
	}
	else {
	    ps.println("FALSE");		
	}
    }    
}

		
